# Authours: Daniel Cairns, Eyre Hong, Bruce Wu, Zilong Yi (UBC MDS)
# Date: Jan 14th, 2023

import pandas as pd
import numpy as np
import jellyfish as jf
import random

def find_name(sex, init, length):
    """
    Generate the a random set of 10 suggested baby names based on the given limitations.
    Parameters
    ----------
    sex: string, 
        The sex of baby's name, either 'F' or 'M' or 'f' or 'm'
    init : string
        The initial of baby's name, a single alphabet character
    length : int
        The length of baby's name, larager than 0
    Returns
    -------
    list:
        A name list containing random suggested names based on the given limitation.
    Examples
    --------
    >>> find_name('F', 'A', 3)
    >>> ['Ada', 'Aja', 'Ani', 'Aya', 'Ava', 'Ana', 'Ari', 'Ali', 'Ami', 'Amy']
    """
    # Check input type of sex
    if not isinstance(sex, str):
        raise TypeError("sex needs to be a str type.")
    
    # Check input type of init
    if not isinstance(init, str):
        raise TypeError("init needs to be a str type.")

    # Check input type of length
    if not isinstance(length, int):
        raise TypeError("length needs to be an int type.")
    
    # Check input value of sex
    if not (sex == 'F' or sex == 'M' or sex == 'f' or sex == 'm'):
        raise Exception("sex should be either 'F'/'f' or 'M'/'m'.")
    
    # Check input value of init
    if not (len(init) == 1):
        raise Exception("init should be a single character.")
    
    if not (str.isalpha(init)):
        raise Exception("init should be an alphabet.")
    
    # Check input value of length
    if length <= 0:
        raise Exception("length should be larger than 0.")
    
    # Data loading and cleaning   
    url = "https://raw.githubusercontent.com/rfordatascience/tidytuesday/master/data/2022/2022-03-22/babynames.csv"
    raw_df = pd.read_csv(url)
    raw_df = raw_df[raw_df['n'] >= 100] # Keep only names that had at least 100 births for a single gender in a single year
    
    # Filter data based on the arguments
    df_sex = raw_df.loc[raw_df['sex'] == sex.capitalize()]
    df_init = df_sex.loc[df_sex['name'].str.startswith(init.capitalize(), na=False)]
    df_len = df_init.loc[df_init['name'].str.len() == length]
    
    # Create name list and randomly select 10 names
    name_list = df_len['name'].unique().tolist()
    if len(name_list) < 10:
        sampled = random.sample(name_list, k=len(name_list))
    else:
        sampled = random.sample(name_list, k=10)    
    
    return sampled


def find_old_name(tp,limit=10, sex="uni", seed=None):
    """
    Generate a random set of 10(if there are that many exists) suggested neutral(by default) baby names 
    based on the given time period and sex.
    Parameters
    ----------
    tp : string, default 10
        A string specifying time period from 1880 to 2018
        List of possible qualified inputs:
        ['1880s','1890s','1900s','1910s','1920s','1930s','1940s','1950s','1960s','1970s','1980s','1990s','2000s','2010s']
    limit : float
        The number of name in the output list. 
    sex : string, default "uni"
        The aimed sex of the name in the ouput.
        List of possible qualified inputs:
        ["uni","M","F] 
    seed : int
        if provided, returns the same random set. 
    Returns
    -------
    list:
        A name list containing random suggested old names based on the given limitation.
    Examples
    --------
    >>> find_old_name('1980s', seed=123)
    >>> ['Neng','Rashida','Krister','Garry','Kai','Chezare','Dolores','Tyler','Skylar','Tyrie']
    """
    # Load data
    URL = "https://raw.githubusercontent.com/rfordatascience/tidytuesday/master/data/2022/2022-03-22/babynames.csv"
    data = pd.read_csv(URL)
    # Data wrangling
    data["tp"] = np.select(
        [
            data["year"].between(1880,1890, inclusive="left"),
            data["year"].between(1890,1900, inclusive="left"),
            data["year"].between(1900,1910, inclusive="left"),
            data["year"].between(1910,1920, inclusive="left"),
            data["year"].between(1920,1930, inclusive="left"),
            data["year"].between(1930,1940, inclusive="left"),
            data["year"].between(1940,1950, inclusive="left"),
            data["year"].between(1950,1960, inclusive="left"),
            data["year"].between(1960,1970, inclusive="left"),
            data["year"].between(1970,1980, inclusive="left"),
            data["year"].between(1980,1990, inclusive="left"),
            data["year"].between(1990,2000, inclusive="left"),
            data["year"].between(2000,2010, inclusive="left"),
            data["year"].between(2010,2020, inclusive="left"),
        ],
        [
            '1880s',
            '1890s',
            '1900s',
            '1910s',
            '1920s',
            '1930s',
            '1940s',
            '1950s',
            '1960s',
            '1970s',
            '1980s',
            '1990s',
            '2000s',
            '2010s'
        ]
    )
    # Setting seed.
    if seed is not None:
        np.random.seed(seed)
    if tp not in ['1880s','1890s','1900s','1910s','1920s','1930s','1940s','1950s','1960s','1970s','1980s','1990s','2000s','2010s'] or sex not in ["uni",'M','F']:
        raise Exception("Sorry, please enter valid time periods/sex!")
    
    df = data[data["tp"]==tp]
    if sex == "uni":
        F = df[df["sex"] == 'F']["name"]
        M = df[df["sex"] == "M"]["name"]
        uni_df = list(set(F).intersection(set(M)))
        if len(uni_df) < 10:
            return uni_df
        else:
            return np.random.choice(uni_df, limit, replace=False).tolist()
    else: 
        df = df[df["sex"] == sex]["name"]
        return np.random.choice(df,limit,replace=False).tolist()
    

def find_similar_name(match_name, limit=10):
    """
    Generate a random list of names that sound similar to a given user input name.
    
    Uses Match rating approach algorithm to determine similarity
      
    Parameters
    ----------
    match_name : string
        Name for comparison; output names will sound like this one 
    limit : int
        The number of names in the output list. (default = 10)
    Returns
    -------
    list:
        A name list containing random suggested similar names based on the given name.
    Examples
    --------
    >>> find_similar_name('Elizabeth', 5)
    >>> ['Elisabeth', 'Elliott', 'Ellsworth', 'Emmalynn', 'Ryley']
    """
    
    # Check inputs
    if not isinstance(match_name, str):
        raise TypeError("input name must be a string")

    if not isinstance(limit, int):
        raise TypeError("Limit must be an integer")
    
    # Data loading and cleaning
    url = "https://raw.githubusercontent.com/rfordatascience/tidytuesday/master/data/2022/2022-03-22/babynames.csv"
    raw_df = pd.read_csv(url)
    raw_df = raw_df[raw_df["n"] >= 100] # Keep only names that had at least 100 births for a single gender in a single year
    unique_names = raw_df['name'].unique()
    unique_names = unique_names[unique_names != match_name] # Remove target name from potential outputs
    
    # Calculate targets 
    encodings = [jf.match_rating_codex(n) for n in unique_names]
    target = jf.match_rating_codex(match_name)
    distance = [jf.jaro_winkler_similarity(target, e) for e in encodings]
    
    df = pd.DataFrame(data = [unique_names, encodings, distance]).T
    df.columns = ["name", "encoding", "match_score"]
    df["weight"] = (df["match_score"]*2)**10 # Heavily weigh higher scores
    
    # Sample n names to output, weighted by match_score
    sampled = df.sample(n = limit, weights = "weight")
    sampled = sampled.sort_values(by = "match_score", ascending = False)
    return(sampled["name"].to_list())

def find_unisex_name(bar,limit=10):
    """
    Generate the a random set of 10 suggested neutral baby names 
    based on the given limitation and baby names in the past years.
    Parameters
    ----------
    limit : float
        A float controls the minimum proportion of the neutral names in a single year in the dataframe.
    count : integer
        The length of the output with a default value of 10
    Returns
    -------
    list:
        A word list containing random suggested neutral names based on the given limitation
        If the length of the word list is less than the limitation, it will return all the names in the word list
    Examples
    --------
    >>> limit=10
    >>> find_unisex_name(0.2,limit)
    >>> ['Skylar', 'Azariah', 'Royal', 'Hayden', 'Emerson', 'Rowan', 'Baylor', 'Dakota', 'River', 'Emory']
    """
    
    #check input value
    if not (bar > 0 and bar < 1):
        raise Exception("bar value should within 0 and 1")
    
    # Load data
    url = "https://raw.githubusercontent.com/rfordatascience/tidytuesday/master/data/2022/2022-03-22/babynames.csv"
    data = pd.read_csv(url)
    
    # data wrangling
    bar_data_boy = data[(data['prop'] > bar) & (data['sex'] == 'M')] # select all the boy names with prop greater than bar
    bar_data_girl = data[(data['prop'] > bar) & (data['sex'] == 'F')] # select all the girl names with prop greater than bar
    target_names = []
    for girl_name in set(bar_data_girl['name']):
        if girl_name in set(bar_data_boy['name']):
            target_names.append(girl_name)
    
    # Create name list
    common_unisex_names=['Skylar', 'Azariah', 'Royal', 'Hayden', 'Emerson', 
                         'Rowan', 'Baylor', 'Dakota', 'River', 'Emory','Jessie',
                         'Marion','Jackie','Alva','Ollie','Jodie','Cleo','Kerry']
    if len(target_names) < limit:
        result = target_names + np.random.choice(common_unisex_names, limit-len(target_names), replace=False).tolist()
    else:
        result = np.random.choice(target_names, limit, replace=False).tolist()
    
    return result


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['nameforme']

package_data = \
{'': ['*']}

install_requires = \
['jellyfish>=0.9.0,<0.10.0',
 'numpy>=1.24.1,<2.0.0',
 'pandas>=1.5.3,<2.0.0',
 'pytest>=7.2.1,<8.0.0']

setup_kwargs = {
    'name': 'nameforme',
    'version': '0.0.20',
    'description': 'A package used to generate names.',
    'long_description': '[![ci-cd](https://github.com/UBC-MDS/nameforme/actions/workflows/ci-cd.yml/badge.svg)](https://github.com/UBC-MDS/nameforme/actions/workflows/ci-cd.yml) \n[![docs](https://readthedocs.org/projects/nameforme/badge/?version=main)](https://readthedocs.org/projects/nameforme/badge/?version=main)\n[![codecov](https://codecov.io/gh/UBC-MDS/nameforme/branch/main/graph/badge.svg?token=vKUSBGopdL)](https://codecov.io/gh/UBC-MDS/nameforme)\n![pypi](https://img.shields.io/pypi/v/nameforme)\n[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)\n\n# nameforme\n\nA helper python package that can be used to generate names based on the [`dateset`](https://raw.githubusercontent.com/rfordatascience/tidytuesday/master/data/2022/2022-03-22/babynames.csv). This could be used to come up with baby names, character names, pseudonyms, etc. \n\nSource Data: Contains baby names born in the United States for each year from 1880 to 2017, and the number of children of each sex given each name. Names must appear at least 5 times in each year to be included. (Source: http://www.ssa.gov/oact/babynames/limits.html)\n\nThis package is similar to the existing [names](https://pypi.org/project/names/) package by Trey Hunner (last updated in 2014), however, our uses a more recent dataset (with names up to 2017), and more options for users to customize what type of names to generate, including the ability to generate similar sounding names.\n\nYou can check the online documentation of this package on [readthedocs](https://nameforme.readthedocs.io/en/main/index.html).\n\n## Features\nNote that the name of the functions is not finalized. They are subject to change.\n\nThe package is an assimilation of four independent functions:\n\n- `find_unisex_name`: Generate a random set of 10 suggested neutral baby names based on the given limitation and baby names in the past years.\n\n- `find_old_name`: Generate a random set of 10 suggested neutral(by default) baby names based on the given time period and sex.\n\n- `find_similar_name`: Generate a random set of 10 suggested similar baby names based on the syllable of the input name. \n\n- `find_name`: Generate a random set of 10 suggested baby names based on the given limitations.\n\n## Installation\n\n```bash\n$ pip install nameforme\n```\n\n## Usage\n\nBelow is a basic example of how to use each of the four functions included in this package.\n\n```\n# Load all required package functions\nfrom nameforme.nameforme import find_name\nfrom nameforme.nameforme import find_old_name\nfrom nameforme.nameforme import find_similar_name\nfrom nameforme.nameforme import find_unisex_name\n\n# Generate a random set of 10 suggested baby names based on the given limitations.\n# if the given limitation can match to at least 10 names, a list of 10 names will be provided\nfind_name("F", "A", length=3)\n#if the given limitation can only match less than 10 names, all matched names will be provided\nfind_name("m", "b", length=9)\n\n# Generate a random set of suggested neutral(by default) baby names based on the given time period and sex.\nfind_old_name(\'1980s\', limit=3)\n\n# Generate a random list of names that sound similar to a given user input name.\nfind_similar_name(\'Daniel\', limit=20)\n\n# Generate the a random set of suggested neutral baby names based on the given limitation and baby names in the past years.\nfind_unisex_name(bar=0.02,limit=10)\n```\n\n## Dependencies\n- python = "3.9"\n- numpy = "1.24.1"\n- pandas = "1.5.3"\n- jellyfish = "0.9.0"\n- pytest = "7.2.1"\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`nameforme` was created by Daniel Cairns, Eyre Hong, Bruce Wu, Zilong Yi (UBC MDS). It is licensed under the terms of the MIT license.\n\n## Credits\n\n`nameforme` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Daniel Cairns, Eyre Hong, Bruce Wu, Zilong Yi (UBC MDS)',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
